package com.sun.jndi.toolkit.url;

import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.NamingEnumeration;
import javax.naming.directory.ModificationItem;
import javax.naming.spi.ResolveResult;
import javax.naming.directory.Attributes;
import javax.naming.NamingException;
import javax.naming.spi.DirectoryManager;
import javax.naming.CannotProceedException;
import javax.naming.Name;
import java.util.Hashtable;
import javax.naming.directory.DirContext;

public abstract class GenericURLDirContext extends GenericURLContext implements DirContext
{
    protected GenericURLDirContext(Hashtable<?, ?> hashtable) {
        super(hashtable);
    }
    
    protected DirContext getContinuationDirContext(Name name) throws NamingException {
        Object lookup = this.lookup(name.get(0));
        CannotProceedException cpe = new CannotProceedException();
        cpe.setResolvedObj(lookup);
        cpe.setEnvironment(this.myEnv);
        return DirectoryManager.getContinuationDirContext(cpe);
    }
    
    @Override
    public Attributes getAttributes(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.getAttributes(rootURLContext.getRemainingName());
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public Attributes getAttributes(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.getAttributes(name.get(0));
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.getAttributes(name.getSuffix(1));
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public Attributes getAttributes(String s, String[] array) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.getAttributes(rootURLContext.getRemainingName(), array);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public Attributes getAttributes(Name name, String[] array) throws NamingException {
        if (name.size() == 1) {
            return this.getAttributes(name.get(0), array);
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.getAttributes(name.getSuffix(1), array);
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public void modifyAttributes(String s, int n, Attributes attributes) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            dirContext.modifyAttributes(rootURLContext.getRemainingName(), n, attributes);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public void modifyAttributes(Name name, int n, Attributes attributes) throws NamingException {
        if (name.size() == 1) {
            this.modifyAttributes(name.get(0), n, attributes);
        }
        else {
            DirContext continuationDirContext = this.getContinuationDirContext(name);
            try {
                continuationDirContext.modifyAttributes(name.getSuffix(1), n, attributes);
            }
            finally {
                continuationDirContext.close();
            }
        }
    }
    
    @Override
    public void modifyAttributes(String s, ModificationItem[] array) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            dirContext.modifyAttributes(rootURLContext.getRemainingName(), array);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public void modifyAttributes(Name name, ModificationItem[] array) throws NamingException {
        if (name.size() == 1) {
            this.modifyAttributes(name.get(0), array);
        }
        else {
            DirContext continuationDirContext = this.getContinuationDirContext(name);
            try {
                continuationDirContext.modifyAttributes(name.getSuffix(1), array);
            }
            finally {
                continuationDirContext.close();
            }
        }
    }
    
    @Override
    public void bind(String s, Object o, Attributes attributes) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            dirContext.bind(rootURLContext.getRemainingName(), o, attributes);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public void bind(Name name, Object o, Attributes attributes) throws NamingException {
        if (name.size() == 1) {
            this.bind(name.get(0), o, attributes);
        }
        else {
            DirContext continuationDirContext = this.getContinuationDirContext(name);
            try {
                continuationDirContext.bind(name.getSuffix(1), o, attributes);
            }
            finally {
                continuationDirContext.close();
            }
        }
    }
    
    @Override
    public void rebind(String s, Object o, Attributes attributes) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            dirContext.rebind(rootURLContext.getRemainingName(), o, attributes);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public void rebind(Name name, Object o, Attributes attributes) throws NamingException {
        if (name.size() == 1) {
            this.rebind(name.get(0), o, attributes);
        }
        else {
            DirContext continuationDirContext = this.getContinuationDirContext(name);
            try {
                continuationDirContext.rebind(name.getSuffix(1), o, attributes);
            }
            finally {
                continuationDirContext.close();
            }
        }
    }
    
    @Override
    public DirContext createSubcontext(String s, Attributes attributes) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.createSubcontext(rootURLContext.getRemainingName(), attributes);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public DirContext createSubcontext(Name name, Attributes attributes) throws NamingException {
        if (name.size() == 1) {
            return this.createSubcontext(name.get(0), attributes);
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.createSubcontext(name.getSuffix(1), attributes);
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public DirContext getSchema(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        return ((DirContext)rootURLContext.getResolvedObj()).getSchema(rootURLContext.getRemainingName());
    }
    
    @Override
    public DirContext getSchema(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.getSchema(name.get(0));
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.getSchema(name.getSuffix(1));
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public DirContext getSchemaClassDefinition(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.getSchemaClassDefinition(rootURLContext.getRemainingName());
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public DirContext getSchemaClassDefinition(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.getSchemaClassDefinition(name.get(0));
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.getSchemaClassDefinition(name.getSuffix(1));
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes attributes) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.search(rootURLContext.getRemainingName(), attributes);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes attributes) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), attributes);
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.search(name.getSuffix(1), attributes);
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes attributes, String[] array) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.search(rootURLContext.getRemainingName(), attributes, array);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes attributes, String[] array) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), attributes, array);
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.search(name.getSuffix(1), attributes, array);
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, String s2, SearchControls searchControls) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.search(rootURLContext.getRemainingName(), s2, searchControls);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String s, SearchControls searchControls) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), s, searchControls);
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.search(name.getSuffix(1), s, searchControls);
        }
        finally {
            continuationDirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, String s2, Object[] array, SearchControls searchControls) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        DirContext dirContext = (DirContext)rootURLContext.getResolvedObj();
        try {
            return dirContext.search(rootURLContext.getRemainingName(), s2, array, searchControls);
        }
        finally {
            dirContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String s, Object[] array, SearchControls searchControls) throws NamingException {
        if (name.size() == 1) {
            return this.search(name.get(0), s, array, searchControls);
        }
        DirContext continuationDirContext = this.getContinuationDirContext(name);
        try {
            return continuationDirContext.search(name.getSuffix(1), s, array, searchControls);
        }
        finally {
            continuationDirContext.close();
        }
    }
}
